@@ -1,56 +1,48 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 use Exporter 'import';
-our @EXPORT_OK = qw( portrait generate landscape transpose flip mirror reverse earthquake tsunami );
+our @EXPORT_OK = qw( portrait generate landscape tornado flip mirror reverse earthquake tsunami );
 
 use Clone;
 use HTML::AutoTag;
 use Math::Matrix;
 use Spreadsheet::HTML::File::Loader;
 
-sub portrait    { generate( @_ ) }
-sub generate    { _make_table( _process( @_ ) ) }
+sub portrait    { generate( @_, theta =>   0 ) }
+sub landscape   { generate( @_, theta => -270, tgroups => 0 ) }
 
-sub landscape   { transpose( @_ ) }
-sub transpose   {
-    my %args = _process( @_ );
-    $args{data} = [@{ Math::Matrix::transpose( $args{data} ) }];
-    return _make_table( %args, tgroups => 0 );
-}
+sub north   { generate( @_, theta =>    0 ) }
+sub east    { generate( @_, theta =>   90, tgroups => 0 ) }
+sub south   { generate( @_, theta => -180, tgroups => 0 ) }
+sub west    { generate( @_, theta => -270, tgroups => 0 ) }
 
-sub flip   {
+sub generate    {
     my %args = _process( @_ );
-    $args{data} = [ CORE::reverse @{ $args{data} } ];
-    return _make_table( %args, tgroups => 0 );
-}
 
-sub mirror   {
-    my %args = _process( @_ );
-    $args{data} = [ map [ CORE::reverse @$_ ], @{ $args{data} } ];
+    $args{theta} *= -1 if $args{flip};
+
+    if (!$args{theta}) {
+        $args{data} = $args{flip} ? [ map [ CORE::reverse @$_ ], @{ $args{data} } ] : $args{data};
+    } elsif ($args{theta} == -90) {
+        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }]
+    } elsif ($args{theta} == 90) {
+        $args{data} = [ map [ CORE::reverse @$_ ], @{ Math::Matrix::transpose( $args{data} ) }];
+    } elsif ($args{theta} == -180) {
+        $args{data} = [ CORE::reverse @{ $args{data} } ]
+    } elsif ($args{theta} == 180) {
+        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
+    } elsif ($args{theta} == -270) {
+        $args{data} = [@{ Math::Matrix::transpose( $args{data} ) }]
+    } elsif ($args{theta} == 270) {
+        $args{data} = [ CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
+    }
+
     return _make_table( %args );
 }
 
-sub reverse   {
-    my %args = _process( @_ );
-    $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
-    return _make_table( %args, tgroups => 0 );
-}
-
-sub earthquake   {
-    my %args = _process( @_ );
-    $args{data} = [ map [ CORE::reverse @$_ ], @{ Math::Matrix::transpose( $args{data} ) }];
-    return _make_table( %args, tgroups => 0 );
-}
-
-sub tsunami   {
-    my %args = _process( @_ );
-    $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
-    return _make_table( %args, tgroups => 0 );
-}
-
 sub new {
     my $class = shift;
     my %attrs = ref($_[0]) eq 'HASH' ? %{+shift} : @_;
@@ -76,15 +68,15 @@ sub _process {
         }
     }
 
-    # headings is an alias for row_0
-    $args->{-row_0} = delete $args->{headings} if exists $args->{headings};
+    # headings is an alias for row0
+    $args->{-row0} = delete $args->{headings} if exists $args->{headings};
 
     # headings to index mapping for column
     my %index = ();
     if ($#{ $data->[0] }) {
         %index = map { '-' . $data->[0][$_] || '' => $_ } 0 .. $#{ $data->[0] };
         for (grep /^-/, keys %$args) {
-            $args->{"-col_$index{$_}" } = delete $args->{$_} if exists $index{$_};
+            $args->{"-col$index{$_}" } = delete $args->{$_} if exists $index{$_};
         }
     }
 
@@ -99,23 +91,23 @@ sub _process {
             my $tag = (!$row and !($args->{headless} or $args->{matrix})) ? 'th' : 'td';
             my $val = $data->[$row][$col];
 
-            # -row_X
+            # -rowX
             my $attr = $args->{$tag};
-            if (exists $args->{"-row_$row"}) {
-                if (ref($args->{"-row_$row"}) eq 'CODE') {
-                    $val = $args->{"-row_$row"}->($val);
-                } elsif (ref($args->{"-row_$row"}) eq 'HASH') {
-                    $attr = $args->{"-row_$row"};
+            if (exists $args->{"-row$row"}) {
+                if (ref($args->{"-row$row"}) eq 'CODE') {
+                    $val = $args->{"-row$row"}->($val);
+                } elsif (ref($args->{"-row$row"}) eq 'HASH') {
+                    $attr = $args->{"-row$row"};
                 }
             }
 
-            # -col_X
+            # -colX
             unless (!$args->{matrix} and $row == 0) {
-                if (exists $args->{"-col_$col"}) {
-                    if (ref($args->{"-col_$col"}) eq 'CODE') {
-                        $val = $args->{"-col_$col"}->($val);
-                    } elsif (ref($args->{"-col_$col"}) eq 'HASH') {
-                        $attr = $args->{"-col_$col"};
+                if (exists $args->{"-col$col"}) {
+                    if (ref($args->{"-col$col"}) eq 'CODE') {
+                        $val = $args->{"-col$col"}->($val);
+                    } elsif (ref($args->{"-col$col"}) eq 'HASH') {
+                        $attr = $args->{"-col$col"};
                     }
                 }
             }
@@ -210,7 +202,18 @@ sub _args {
         $args = {@_};
         $data = delete $args->{data} if exists $args->{data};
     } elsif (@_ > 1 && ref($_[0]) eq 'ARRAY') {
-        $data = [ @_ ];
+        if (ref($_[0]->[0]) eq 'ARRAY') {
+            $data = shift;
+        }
+        my @args;
+        for (@_) {
+            if (ref($_) eq 'ARRAY') {
+                push @$data, $_;
+            } else {
+                push @args, $_; 
+            }
+        }
+        $args = {@args};
     } elsif (@_ == 1) {
         $data = $_[0];
     }
@@ -230,6 +233,13 @@ sub _args {
     return ( $self, Clone::clone($data), $args );
 }
 
+sub transpose   { no warnings; warn "transpose is deprecated, use landscape"; generate( @_, theta => -270, tgroups => 0 ) }
+sub earthquake  { no warnings; warn "earthquake is deprecated, use east"; generate( @_, theta =>  90, tgroups => 0 ) }
+sub reverse     { no warnings; warn "reverse is deprecated, use south with flip"; generate( @_, theta => 180, tgroups => 0 ) }
+sub mirror      { no warnings; warn "mirror is deprecated, use portrait with flip"; generate( @_, theta =>    0, flip => 1 ) }
+sub tsunami     { no warnings; warn "tsunami is deprecated, use east with flip"; generate( @_, theta =>  -90, tgroups => 0 ) }
+sub flip        { no warnings; warn "flip is deprecated, use south"; generate( @_, theta => -180, tgroups => 0 ) }
+
 
 1;
 
@@ -267,14 +277,9 @@ what tags and attributes to use.
 
 =head1 METHODS
 
-All methods (except C<new>) are exportable as functions too. With the
-exception of C<new>, all methods return HTML as a scalar string. Any
-named parameters supplied to the method are applied to the data before
-any table rotations are performed. If it helps, work with the table
-using, for example, C<portrait()> until you have the parameters
-correct and then switch to C<landscape()> for the final product.
-All methods accept the same named parameters, although some methods
-override certain ones, hopefully in ways that make sense.
+All methods (except C<new>) are exportable as functions. With the
+exception of C<new>, all methods return HTML as a scalar string and
+they accept the same named parameters (see PARAMETERS below).
 
 =over 4
 
@@ -282,58 +287,61 @@ override certain ones, hopefully in ways that make sense.
 
   my $table = Spreadsheet::HTML->new( data => $data );
 
-Constructs object. Accepts named parameters (see PARAMETERS).
-Unless you give it a list of array refs. Or an array ref
-of array refs. Otherwise it expects named parameters. The
-most favorite being 'data' which is exactly an array ref
-of array refs. The first row will be treated as the headings
-unless you specify otherwise (see PARAMETERS).
+Constructs object. Accepts the same named parameters as the table
+generating functions below:
 
 =item * C<generate( %args )>
 
   $html = $table->generate( table => {border => 1}, encode => '<>' );
   print Spreadsheet::HTML::generate( data => $data, indent => "\t" );
 
-Returns a string that contains the rendered HTML table.
-
 =item * C<portrait( %args )>
 
-Alias for C<generate()>
+=item * C<north( %args )>
 
-=item * C<transpose( %args )>
-
-Uses Math::Matrix to rotate the headings and data
-90 degrees counter-clockwise.
+Headers on top C<generate( 'theta', 0 )>
 
 =item * C<landscape( %args )>
 
-Alias for C<transpose()>
+=item * C<west( %args )>
 
-=item * C<flip( %args )>
+Headers on left: C<generate( 'theta', -270 )>
 
-Flips the headings and data upside down.
+=item * C<south( %args )>
 
-=item * C<mirror( %args )>
+Headers on bottom: C<generate( 'theta', -180 )>
 
-Columns are rendered right to left.
+=item * C<east( %args )>
 
-=item * C<reverse( %args )>
+Headers on right: C<generate( 'theta', 90 )>
 
-Combines flip and mirror: flips the headings and
-data upside down and render columns right to left.
+=item * C<transpose( %args )>
+
+Deprecated: use C<landscape()>
 
 =item * C<earthquake( %args )>
 
-Combines transpose/landscape with mirror.
+Deprecated: use C<east()>
 
 =item * C<tsunami( %args )>
 
-Combines transpose/landscape with reverse.
+Deprecated: use C<east( 'flip', 1 )>
+
+=item * C<mirror( %args )>
+
+Deprecated: use C<north( 'flip', 1 )>
+
+=item * C<reverse( %args )>
+
+Deprecated: use C<south( 'flip', 1 )>
+
+=item * C<flip( %args )>
+
+Deprecated: use C<south()>
 
 =back
 
-For most cases, C<portrait()> and C<landscape()> are all you need. They
-are simply aliases for C<generate()> and C<transpose()>, respectively.
+For most cases, C<portrait()> and C<landscape()> are all you need.
 
 =head1 PARAMETERS
 
@@ -361,6 +369,17 @@ are XLS, CSV, JSON, YAML and HTML (first table found).
 
   file => 'foo.json'
 
+=item * C<theta: 0, 90, 180, 270, -90, -180, -270>
+
+Rotates table clockwise. Default to 0: headers at top.
+90: headers at right. 180: headers at bottom.
+270: headers at left. To achieve landscape, use -270.
+
+=item * C<flip: 0 or 1>
+
+Flips table horizontally.
+Can be used in conjunction with C<theta>.
+
 =item * C<indent: $str>
 
 Render the table with nested indentation. Defaults to
@@ -369,7 +388,7 @@ when set to any value that is defined.
 
   indent => '    '
 
-=item * C<encode: $str>
+=item * C<encodes: $str>
 
 HTML Encode contents of td tags. Defaults to empty string
 which performs no encoding of entities. Pass a string like
@@ -424,25 +443,25 @@ Or apply hash ref as attributes:
 
   headings => { class => 'some-class' }
 
-=item * C<-row_X: \& or \%>
+=item * C<-rowX: \& or \%>
 
 Apply this anonymous subroutine to row X. (0 index based)
 
-  -row_3 => sub { uc shift }
+  -row3 => sub { uc shift }
 
 Or apply hash ref as attributes:
 
-  -row_3 => { class => 'some-class' }
+  -row3 => { class => 'some-class' }
 
-=item * C<-col_X: \& or \%>
+=item * C<-colX: \& or \%>
 
 Apply this anonymous subroutine to column X. (0 index based)
 
-  -col_4 => sub { sprintf "%02d", shift || 0 }
+  -col4 => sub { sprintf "%02d", shift || 0 }
 
 Or apply hash ref as attributes:
 
-  -col_4 => { class => 'some-class' }
+  -colo => { class => 'some-class' }
 
 You can alias any column number by the value of the heading
 name in that column:
@@ -573,6 +592,8 @@ value contol, rotating attributes and totals/subtotals.
 
 =item * L<http://www.w3.org/TR/html5/tabular-data.html>
 
+=item * L<http://en.wikipedia.org/wiki/Rotation_matrix>
+
 =back
 
 =head1 BUGS AND LIMITATIONS
@@ -621,15 +642,6 @@ Please report any bugs or feature requests to either
 I will be notified, and then you'll automatically be notified of progress
 on your bug as I make changes.
 
-In terms of limitations this implementation is not as fast as it should be.
-From the results a few performance tests, i believe this to be blamed on HTML::Tree.
-v0.02 could process a 500x500 data matrix in 1/5 of a second. This version 
-timed at around 8.5 seconds. Not awesome. DBIx::XHTML_Table timed at 2.2 seconds.
-The several lines of code that HTML::Element save me are not worth the time
-trade off, so i will be working to develop my own solution, unless another CPAN
-module will suffice. Don't get me wrong, HTML::Tree is awesome and powerful.
-But i needs speed. And i could be wrong ... ;)
-
 =head1 GITHUB
 
 The Github project is L<https://github.com/jeffa/Spreadsheet-HTML>
